︠1cedd069-bb38-456d-a7c1-8b1e56d5163c︠
# C : periodic coupon (interest rate) [ex, 5% would be .05]
# n : number of periods [ex, 10]
# F : maturity value [ex, 1000]
# r : required yield [ex, 5% would be .05]
def compute_units(C, n, F, r):
    return C * F * ((1 - (1 / (1 + r) ^ n)) / r) + F * (1 / (1 + r) ^ n)

# Same as compute_units, except returns a scaled price (100 being par)
# instead of the units.
def compute_price(C, n, F, r):
    return 100 * (compute_units(C, n, F, r) / F)
︡d8819093-9861-4e46-bf88-db3096ecb00e︡︡
︠206db9c4-b862-4b91-a911-aa2a6cd0a367︠
# Graph a price yield curve.  7% coupon, $1000 principal, with yield ranging from 0% to 20%.
# The red curve is a 10 year bond, and the blue is a 1 year bond.
p = plot(compute_price(.07,1,1000,var('r')),0,.2)
p += plot(compute_price(.07,10,1000,var('r')),0,.2, rgbcolor=(1,0,0))
show(p)
︡3dfa229d-5cc6-48f1-81c0-2c26ec423fcd︡︡
︠725b6d5c-4abd-4965-b3ab-9017082de464︠
# Same scenario as before, but showing the time periods as a 3rd dimension instead of
# overlaying the curves.  Note: the previous 2 curves are the endpoints of this surface.
plot3d(compute_price(.07,var('n'),1000,var('r')), (n,1,10), (r,0,.2))
︡89154b9c-329d-4889-ab7c-72d64e1597ee︡︡
︠db00bb23-1521-45d3-b711-eaec48f86613︠
# Newton-Raphson function.  Takes a function, its derivative, an initial guess, and 
# an optional tolerance.
def newton(func, funcd, x, TOL=1e-6):
    f, fd = func(x), funcd(x)
    count = 0
    while 1:
        dx = f / float(fd)
        if abs(dx) < TOL * (1 + abs(x)): return x - dx
        x = x - dx
        f, fd = func(x), funcd(x)
        count = count + 1
        print "newton(%d): x=%s, f(x)=%s" % (count, x, f)
︡48680df6-25e8-46f3-a931-cafb6d5f970a︡︡
︠941cf021-42b5-4392-8e13-22a0ce5fd514︠
# Compute the yield given the bond characteristics, a price, and an initial guess.
def compute_yield(C, n, F, u, y0):
    r = var('r')
    func = compute_units(C,n,F,r) - u
    funcd = diff(func, r)
    return newton(func, funcd, y0)
︡089cc1e9-5f43-4d6d-9ddf-29d3e4686173︡︡
︠777eef8f-002f-42fb-b1fa-2a4c48f0ce04︠
C = .0675   # coupon
n = 20      # periods
F = 10000   # principal
u = 9125    # bond price (units)
g = .1      # initial guess for yield

# This is like computing f(g(f(x))), where f computes units and g computes yield.
compute_units(C, n, F, compute_yield(C, n, F, u, .1))
︡f3ad2ff4-ea48-4019-afee-31af2d7a8216︡{"stdout": "newton(1): x=\n                              .07144291315814208, f(x)=\n                               461.9310522332926\nnewton(2): x=\n                              .07599073225081789, f(x)=\n                               15.89642112795082\nnewton(3): x=\n                              .07615863583051595, f(x)=\n                              .02048624306917191\n9125.00000003"}︡
